// Copyright (c) Meta Platforms, Inc. and affiliates.
// All rights reserved.
//
// This source code is licensed under the BSD-style license found in the
// LICENSE file in the root directory of this source tree.

#pragma once

#include <cstddef>
#include <memory>
#include <optional>
#include <utility>
#include <vector>

#include "fairseq2n/data/data_pipeline.h"
#include "fairseq2n/data/data_source.h"

namespace fairseq2n::detail {

class map_data_source final : public data_source {
public:
    explicit
    map_data_source(
        std::unique_ptr<data_source> &&inner, map_fn &&fn, std::size_t num_parallel_calls);

    std::optional<data>
    next() override;

    void
    reset() override;

    void
    record_position(tape &t) const override;

    void
    reload_position(tape &t) override;

private:
    bool
    fill_buffer();

    std::optional<data>
    invoke_function(data &&example);

private:
    std::unique_ptr<data_source> inner_;
    map_fn map_fn_;
    std::size_t num_parallel_calls_;
    std::vector<std::optional<data>> buffer_{};
    std::vector<std::optional<data>>::iterator buffer_iter_{};
};

}  // namespace fairseq2n::detail
